/*
* Copyright (c) 2017 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free,
* non-exclusive license under copyrights and patents it now or hereafter
* owns or controls to make, have made, use, import, offer to sell and sell ("Utilize")
* this software subject to the terms herein.  With respect to the foregoing patent
* license, such license is granted  solely to the extent that any such patent is necessary
* to Utilize the software alone.  The patent license shall not apply to any combinations which
* include this software, other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license (including the
* above copyright notice and the disclaimer and (if applicable) source code license limitations below)
* in the documentation and/or other materials provided with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided that the following
* conditions are met:
*
*             * No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any
*               software provided in binary form.
*             * any redistribution and use are licensed by TI for use only with TI Devices.
*             * Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the source code are permitted
* provided that the following conditions are met:
*
*   * any redistribution and use of the source code, including any resulting derivative works, are licensed by
*     TI for use only with TI Devices.
*   * any redistribution and use of any object code compiled from the source code and any resulting derivative
*     works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or
* promote products derived from this software without specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/
/*
 *  ======== spiSlave_MSP432.c ========
 *  SPI Data Transfer: IR1443 Master to MSP432 Slave
 *
 *  Pin Assignments:
 *  |        |  IR1443   |   MSP432  |
 *  |SPI_CLK |   J6.13   |   P1.5    |
 *  |SPI_MOSI|   J5.12   |   P1.6    |
 *  |SPI_MISO|   J5.14   |   P1.7    |
 */
#include <stdio.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>

/* POSIX Header files */
#include <pthread.h>

/* Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>
#include <ti/drivers/I2C.h>
#include <ti/drivers/Power.h>
#include <ti/drivers/power/PowerMSP432.h>
#include <ti/drivers/dpl/SemaphoreP.h>

/* Example/Board Header files */
#include "Board.h"
#include "msp_demo.h"



//for USLEEP:
#include <unistd.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>




#define THREADSTACKSIZE                768

/* Externed Globals */
extern volatile uint32_t timer_ticks;
extern volatile uint32_t timer_event;
extern volatile uint16_t uart_rx_cmd;
extern SemaphoreP_Handle uart_semHandle;

/* Globals used here only */
uint32_t program_mode;
uint32_t measurement_count;
uint32_t reentrancy_check;








#ifdef REPORT_IR14_TIME_MEASUREMENTS
uint32_t meas_time_start;
uint32_t meas_time_boot;
uint32_t meas_time_cal;
uint32_t meas_time_config;
uint32_t meas_time_proc;
uint32_t meas_time_done;
#endif

/* Demo Configuration Parameters */
MSP_Demo_Parms demo_parms =
{
  LEVEL_SENSE_EVENTS_PER_SEC,          // frequency of wake up events
  NUM_IR1443_RESULTS_PER_MEAS,         // expected radar results per measurement
  0                                    // wake up rate in timer clocks
};


/* This function initializes non-driver related items for the demo. */
void demo_init(void)
{
  uint32_t clks;

  clks = MSP432_TIMER_TICKS_PER_SEC / demo_parms.wakeup_rate;
  demo_parms.us_per_wakeup = clks;

  if ((clks * demo_parms.wakeup_rate) != MSP432_TIMER_TICKS_PER_SEC)
    demo_parms.us_per_wakeup ++;

  reentrancy_check = 0;
  measurement_count = 0;
  program_mode = MSP_DEMO_MODE_STEP;
}


/* This function controls the process of taking a single measurement from the
 * IR1443 device.  It will block until the process is complete. The steps are:
 *
 *   1) Release the IR1443 from reset.
 *   2) Set a GPIO high for power measurement to begin.
 *   3) Block until IR1443 data is received in the SPI Rx.
 *   4) Set a GPIO low for power measurement to stop.
 *   5) Put the IR1443 back into reset.
 */
LSDemo_ChirpData *perform_measurement(void)
{
  LSDemo_ChirpData *data;
  int32_t  code;
  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  int32_t  err;
  uint32_t t_diff;
  float    t_ms;
  #endif
  char     tmpbuf[64];


  //If the run mode timer is enabled and it is set to a rate faster than the
  //IR1443 can complete a boot+measurement cycle, then this function may be
  //called reentrantly. If this happens, just return and do nothing.
  if (reentrancy_check)
    return(NULL);

  code = 0;
  reentrancy_check = 1;

  //Step 1: Power on the IR1443.  This sets GPIO 4.7 high, which can be monitored
  //by test equipment as the point at which the IR device is given power.
  gpio_release_ir1443_reset();
  // Added for I2C bringing radar out of reset
  gpio_nRST_set_float();



  //Step 2: If time measurement is required, see how long boot took
  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  //First, grab a timer value now that we've turned the IR14xx on.
  meas_time_start = timer_get_timer32();

  //Wait on IR14xx GPIO 0 to go high, marking that IWR boot is complete.
  err = gpio_wait_on_ir14xx_gpio0(1);
  if (err < 0)
  {
    code = -100;
    goto Abort;
  }

  //Grab the timer again to find the difference.
  meas_time_boot = timer_get_timer32();

  //Wait on IR14xx GPIO 0 to go low, marking that IWR config is complete.
  err = gpio_wait_on_ir14xx_gpio0(0);
  if (err < 0)
  {
    code = -101;
    goto Abort;
  }

  //Grab the timer again to find the difference.
  meas_time_config = timer_get_timer32();

  //Wait on IR14xx GPIO 0 to go high, marking that IWR chirp processing is complete.
  err = gpio_wait_on_ir14xx_gpio0(1);
  if (err < 0)
  {
    code = -102;
    goto Abort;
  }

  //Grab the timer again to find the difference.
  meas_time_proc = timer_get_timer32();
  #endif


  //Step 3: Block until IR1443 sends measurement data via SPI.
  /* SPI Transfer will block until the burst is received */
  data = (LSDemo_ChirpData *)spi_get_transfer(MSP_SPI_MSG_LENGTH);
  if (data == NULL)
    code = -103;
  else
    code = (int32_t)data->err;


  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  //Grab the timer value again now that we've turned the IR14xx off.
  meas_time_done = timer_get_timer32();

  Abort:
  #endif

  //Step 4: Remove power from the IR1443.
  gpio_enter_ir1443_reset();


  #ifdef REPORT_IR14_TIME_MEASUREMENTS

  if (err == 0)
  {
    //Report the boot time to UART
    if (meas_time_start >= meas_time_boot)
      t_diff = meas_time_start - meas_time_boot;
    else
      t_diff = meas_time_start + (TIMER32_LOAD_VAL - meas_time_boot);

    t_ms   = (float)t_diff / MSP432_CLOCKS_PER_MS;  //convert xMhz clocks to microseconds
    sprintf(tmpbuf, "IWR Boot time: %.2fms (%d clks)", t_ms, t_diff);
    uart_send_string(tmpbuf, UART_LF);

    //Report the configuration time to UART.
    if (meas_time_boot >= meas_time_config)
      t_diff = meas_time_boot - meas_time_config;
    else
      t_diff = meas_time_boot + (TIMER32_LOAD_VAL - meas_time_config);

    t_ms   = (float)(t_diff / MSP432_CLOCKS_PER_MS);  //convert xMhz clocks to microseconds
    sprintf(tmpbuf, "IWR Config time: %.2fms (%d clks)", t_ms, t_diff);
    uart_send_string(tmpbuf, UART_LF);

    //Report the chirp processing time (and total On time) to UART.
    if (meas_time_config >= meas_time_proc)
      t_diff = meas_time_config - meas_time_proc;
    else
      t_diff = meas_time_config + (TIMER32_LOAD_VAL - meas_time_proc);

    t_ms   = (float)(t_diff / MSP432_CLOCKS_PER_MS);  //convert xMhz clocks to microseconds
    sprintf(tmpbuf, "IWR Chirp time: %.2fms (%d clks)", t_ms, t_diff);
    uart_send_string(tmpbuf, UART_LF);

    //Report the chirp processing time (and total On time) to UART.
    if (meas_time_proc >= meas_time_done)
      t_diff = meas_time_proc - meas_time_done;
    else
      t_diff = meas_time_proc + (TIMER32_LOAD_VAL - meas_time_done);

    t_ms   = (float)(t_diff / MSP432_CLOCKS_PER_MS);  //convert xMhz clocks to microseconds
    sprintf(tmpbuf, "IWR Post time: %.2fms (%d clks)", t_ms, t_diff);
    uart_send_string(tmpbuf, UART_LF);
  }
  #endif

  if (code < 0)
  {
    switch (code)
    {
      case DEMO_MEAS_ERR_CONFIG:
        uart_send_string("Measurement error (MMWave_Config error)", UART_LF);
        break;
      case DEMO_MEAS_ERR_START:
        uart_send_string("Measurement error (MMWave_start failure)", UART_LF);
        break;
      case DEMO_MEAS_ERR_FFT:
        uart_send_string("Measurement error (FFT did not return)", UART_LF);
        break;
    #ifdef REPORT_IR14_TIME_MEASUREMENTS
      case -100:
        uart_send_string("Measurement timeout (Boot)", UART_LF);
        break;
      case -101:
        uart_send_string("Measurement timeout (Config)", UART_LF);
        break;
      case -102:
        uart_send_string("Measurement timeout (Chirp)", UART_LF);
        break;
    #endif
      case -103:
        uart_send_string("SPI Transfer error", UART_LF);
        break;
      default:
        sprintf(tmpbuf, "Measurement error (unknown error 0x%8.8x)", code);
        uart_send_string(tmpbuf, UART_LF);
        break;
    }

    data = NULL;
  }

  measurement_count++;
  reentrancy_check = 0;
  return(data);
}


LSDemo_CalData *perform_calibration(void)
{
  LSDemo_CalData *data;
  int32_t  err;
  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  uint32_t t_diff;
  float    t_ms;
  #endif
  char     tmpbuf[64];

  gpio_set_gpio1(MSP_DEMO_IWR1443_CALIBRATE);

  //Power on the IR1443.  This sets GPIO 4.7 high, which can be monitored
  //by test equipment as the point at which the IR device is given power.
  gpio_release_ir1443_reset();

  // added for bringing radar out of reset
  gpio_nRST_set_float();

  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  //First, grab a timer value now that we've turned the IR14xx on.
  meas_time_start = timer_get_timer32();

  //Wait on IR14xx GPIO 0 to go high, marking that MSS app is running.
  err = gpio_wait_on_ir14xx_gpio0(1);
  if (err < 0)
  {
    err  = -100;
    data = NULL;
    meas_time_boot = meas_time_start;
    meas_time_cal  = meas_time_start;
    meas_time_done = meas_time_start;
    goto Abort;
  }

  //Grab the timer again to find the difference.
  meas_time_boot = timer_get_timer32();

  //Wait on IR14xx GPIO 0 to go low, marking that calibration is done.
  err = gpio_wait_on_ir14xx_gpio0(0);
  if (err < 0)
  {
    err  = -101;
    data = NULL;
    meas_time_boot = meas_time_start;
    meas_time_cal  = meas_time_start;
    meas_time_done = meas_time_start;
    goto Abort;
  }

  //Grab the timer again to find the difference.
  meas_time_cal = timer_get_timer32();
  #endif

  data = (LSDemo_CalData *)spi_get_transfer(MSP_SPI_CAL_LENGTH);
  if (data == NULL)
    err  = -102;

  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  //Grab the timer value again now that we've turned the IR14xx off.
  meas_time_done = timer_get_timer32();

  Abort:
  #endif

  //Remove power from the IR1443.
  gpio_enter_ir1443_reset();

  //Reset IWR1443 GPIO_1 to zero
  gpio_set_gpio1(0);

  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  if (err == 0)
  {
    if ((data == NULL) || (data->pass != DEMO_MAGIC_CAL_PASS))
    {
      meas_time_boot = meas_time_start;
      meas_time_done = meas_time_start;
    }

    //Report the boot time to UART
    if (meas_time_start >= meas_time_boot)
      t_diff = meas_time_start - meas_time_boot;
    else
      t_diff = meas_time_start + (TIMER32_LOAD_VAL - meas_time_boot);

    t_ms   = (float)t_diff / MSP432_CLOCKS_PER_MS;  //convert xMhz clocks to microseconds
    sprintf(tmpbuf, "IWR Boot time: %.2fms (%d clks)", t_ms, t_diff);
    uart_send_string(tmpbuf, UART_LF);

    //Report the configuration time to UART.
    if (meas_time_boot >= meas_time_cal)
      t_diff = meas_time_boot - meas_time_cal;
    else
      t_diff = meas_time_boot + (TIMER32_LOAD_VAL - meas_time_cal);

    t_ms   = (float)(t_diff / MSP432_CLOCKS_PER_MS);  //convert xMhz clocks to microseconds
    sprintf(tmpbuf, "IWR Calibration time: %.2fms (%d clks)", t_ms, t_diff);
    uart_send_string(tmpbuf, UART_LF);

    //Report the configuration time to UART.
    if (meas_time_cal >= meas_time_done)
      t_diff = meas_time_cal - meas_time_done;
    else
      t_diff = meas_time_cal + (TIMER32_LOAD_VAL - meas_time_done);

    t_ms   = (float)(t_diff / MSP432_CLOCKS_PER_MS);  //convert xMhz clocks to microseconds
    sprintf(tmpbuf, "IWR Flash+SPI time: %.2fms (%d clks)", t_ms, t_diff);
    uart_send_string(tmpbuf, UART_LF);
  }
  #endif

  uart_send_string("", UART_LF);

  if (data == NULL)
  {
    sprintf(tmpbuf, "Calibration FAILED (unknown error %d)", err);
    uart_send_string(tmpbuf, UART_LF);
  }
  else
  {
    switch (data->pass)
    {
      case DEMO_MAGIC_CAL_PASS: //magic value found
        uart_send_string("Calibration Successful", UART_LF);
        break;
      case -1:
        uart_send_string("Calibration FAILED (MMWave_initRF)", UART_LF);
        break;
      case -2:
        uart_send_string("Calibration FAILED (rlRfCalibDataStore)", UART_LF);
        break;
      case -3:
        uart_send_string("Calibration FAILED (Invalid Cal data)", UART_LF);
        break;
      case -4:
        sprintf(tmpbuf, "Calibration FAILED (task fail, mask = 0x%4.4x)", data->taskMask);
        uart_send_string(tmpbuf, UART_LF);
        break;
      default:
        sprintf(tmpbuf, "Calibration FAILED (unknown return code %d)", data->pass);
        uart_send_string(tmpbuf, UART_LF);
        break;
    }
  }

  return(data);
}


void output_measurement(LSDemo_ChirpData *chirp_data)
{
  uint32_t idx;
  char     tmpbuf[64];

  //Step 6: Output range data to the UART.
  if (chirp_data != NULL)
  {
    uart_send_string("", UART_LF);
    uart_send_string("IWR1443 Data:", UART_LF);

    for (idx = 0; idx < DEMO_NUM_RESULTS; idx++)
    {
      sprintf(tmpbuf, "Dist  %f(m), power %d",
              chirp_data->distance[idx], chirp_data->power_sqr[idx]);
      uart_send_string(tmpbuf, UART_LF);
    }
  }
}

extern void *uart_thread(void *arg0);

void create_uart_thread()
{
  pthread_t           u_thread;
  pthread_attr_t      pAttrs;
  struct sched_param  priParam;
  int                 retc;
  int                 detachState;

  /* Create application threads */
  pthread_attr_init(&pAttrs);

  detachState = PTHREAD_CREATE_DETACHED;
  /* Set priority and stack size attributes */
  retc = pthread_attr_setdetachstate(&pAttrs, detachState);
  if (retc != 0) {
      /* pthread_attr_setdetachstate() failed */
      while (1);
  }

  retc |= pthread_attr_setstacksize(&pAttrs, THREADSTACKSIZE);
  if (retc != 0) {
      /* pthread_attr_setstacksize() failed */
      while (1);
  }

  priParam.sched_priority = 2;
  pthread_attr_setschedparam(&pAttrs, &priParam);

  /* Create slave thread */
  retc = pthread_create(&u_thread, &pAttrs, uart_thread, (void* )0);
  if (retc != 0) {
      /* pthread_create() failed */
      while (1);
  }
}

void set_PMIC_to_PFM_mode(void)
{
    //CHANGE IWR PMIC MODE TO PFM
    uint8_t         txBuffer[2];
    uint8_t         rxBuffer[47];
    uint8_t         txBufferBUCK2[2];
    uint8_t         txBufferBUCK3[2];
    I2C_Handle      i2c;
    I2C_Params      i2cParams;
    I2C_Transaction i2cTransaction;

    I2C_init();
    gpio_nRST_pull_low();       //Hold radar in reset
    gpio_release_ir1443_reset(); //Bring PMIC_EN high. //JQ: resolving this function vs. AR_NRST function.


    //Check PGood
    uint32_t PGoodStatus = 0;
    PGoodStatus = gpio_read_PGOOD();

    if(PGoodStatus)
    {
        uart_send_string("PGood Check Passed", UART_LF);
    }
    else
    {
        uart_send_string("PGood Check Failed", UART_LF);
    }


    // Wait 1ms
     usleep(10000);


    /* Create I2C for usage */
    I2C_Params_init(&i2cParams);

    i2cParams.bitRate = I2C_400kHz; //default

    i2cParams.transferMode = I2C_MODE_BLOCKING;

    i2c = I2C_open(Board_I2C_TMP, &i2cParams);


    if (i2c == NULL) {
        uart_send_string("MSP432: Error Initializing I2C", UART_LF);
            while (1);
        }
        else {
            uart_send_string("MSP432: I2C Initialized!", UART_LF);
        }

    /* Point to the PMIC and its power ready register */
    i2cTransaction.slaveAddress =  0x60;
    txBuffer[0] = 0x20;
    i2cTransaction.writeBuf = txBuffer;
    i2cTransaction.readBuf = rxBuffer;
    i2cTransaction.readCount = 1;
    i2cTransaction.writeCount = 1;







    do{

           uart_send_string("MSP432: Wait for power good....", UART_LF);
           usleep(1000);
           I2C_transfer(i2c, &i2cTransaction);
           usleep(1000);


       } while(rxBuffer[0] != 0xcc);








    uart_send_string("MSP432: Power good!", UART_LF);
    uart_send_string("MSP432: Ready!", UART_LF);

    txBuffer[0] = 0x01;
    i2cTransaction.writeBuf = txBuffer;
    i2cTransaction.readBuf = rxBuffer;
    i2cTransaction.readCount = 47;
    i2cTransaction.writeCount = 1;
    usleep(10000);

    I2C_transfer(i2c, &i2cTransaction);
    usleep(10000);


    txBuffer[0]      = 0x0E; //BUCK2_VOUT reg
    txBuffer[1]      = 0x89; //Set to 1.3V
    txBufferBUCK2[0] = 0x06; //BUCK2 control reg
    txBufferBUCK2[1] = 0xc4; //BUCK2 set PFM auto
    txBufferBUCK3[0] = 0x08; //BUCK3 control reg
    txBufferBUCK3[1] = 0xc4; //BUCK3 set PFM auto

    i2cTransaction.writeCount = 2;
    i2cTransaction.readCount = 0;
    i2cTransaction.writeBuf = txBuffer;
    I2C_transfer(i2c, &i2cTransaction); //Transmit BUCK2_VOUT voltage





    i2cTransaction.writeBuf = txBufferBUCK2;
    I2C_transfer(i2c, &i2cTransaction); //Transmit BUCK2 settings
    i2cTransaction.writeBuf = txBufferBUCK3;
    I2C_transfer(i2c, &i2cTransaction); //Transmit BUCK3 settings

    //UNCOMMENT FOR DEBUG PURPOSES, READS BACK BUCKS2+3
     txBuffer[0] = 0x01;
     i2cTransaction.writeBuf = txBuffer;
     i2cTransaction.readBuf = rxBuffer;
     i2cTransaction.readCount = 47;
     i2cTransaction.writeCount = 1;
     I2C_transfer(i2c, &i2cTransaction);
     I2C_close(i2c);


}







void initialize(void)
{
  demo_init();
  gpio_init();
  uart_init();
  timer_init();
  spi_init();

#ifdef PMIC_MODE_PFM
  set_PMIC_to_PFM_mode();
  //Initialize IWR1443 GPIO_1 to zero
#endif



    gpio_set_gpio1(0);

  //The default state of GPIO pin P4.7 is off, but initialize it anyway.
  //This will pull PMIC_EN low, removing power from the IR1443.

  gpio_enter_ir1443_reset();

  gpio_set_gpio1(0);
}


/*
 * ======== main application thread ========
 *
 * Here is a listing of the pin usage/connections used for this demo:
 *
 * Use:   IR1443:                MSP432:
 * Info   J6 pin 15 (GPIO 0)     J1 pin 8  (P4.6)
 * Mode   J5 pin 8  (GPIO 1)     J2 pin 4  (P5.7)
 * Reset  J6 pin 16 (PMIC_EN)    J3 pin 8  (P4.7)
 * SPI    J6 pin 13 (IR_SPICLK1) J1 pin 7  (P1.5)
 * SPI    J5 pin 12 (IR_MOSI1)   J2 pin 6  (P1.6)
 * SPI    J5 pin 14 (IR_MISO1)   J2 pin 7  (P1.7)
 * GPIO   n/a                    J2 pin 10 (P3.6, used for power measurement)
 * UART                          usb
 */
void *mainThread(void *arg0)
{
  LSDemo_ChirpData *chirp;

  /* Call driver init functions. */
  initialize();

  create_uart_thread();

  //Send a startup message to the UART (normally connected to TeraTerm)
  uart_send_string("", UART_LF);
  uart_send_string("IWR1443 Level Sensing Demo:", UART_LF);
  uart_send_usage_info();



  /* MSP432 is sleeping when not awakened by an ISR. */
  while (1)
  {
    if (timer_event == 1)
    {
      timer_event = 0;

      chirp = perform_measurement();
      output_measurement(chirp);
      Power_releaseConstraint(PowerMSP432_DISALLOW_SLEEP);
    }
    else
    {
      SemaphoreP_pend(uart_semHandle, SemaphoreP_NO_WAIT);

      if (uart_rx_cmd == UART_RX_CMD_STEP)
      {
        if (program_mode != MSP_DEMO_MODE_RUN)
        {
          uart_send_string("", UART_LF);
          uart_send_string("Start one-shot measurement", UART_LF);
          program_mode = MSP_DEMO_MODE_STEP;
          chirp = perform_measurement();
          output_measurement(chirp);
        }
        else
          uart_send_string("No, the timer is running", UART_LF);

        uart_rx_cmd  = UART_RX_CMD_NONE;
        uart_send_string(">", UART_NO_LF);
      }
      else
      if (uart_rx_cmd == UART_RX_CMD_RUN)
      {
        if (program_mode == MSP_DEMO_MODE_RUN)
        {
          uart_send_string("", UART_LF);
          uart_send_string("Stopping timer", UART_LF);
          timer_stop();
          program_mode = MSP_DEMO_MODE_STEP;
        }
        else
        {
          uart_send_string("", UART_LF);
          uart_send_string("Starting timer", UART_LF);
          timer_start();
          program_mode = MSP_DEMO_MODE_RUN;
        }

        uart_rx_cmd  = UART_RX_CMD_NONE;
        uart_send_string(">", UART_NO_LF);
      }
      else
      if (uart_rx_cmd == UART_RX_CMD_STOP)
      {
        if (program_mode == MSP_DEMO_MODE_RUN)
        {
          uart_send_string("", UART_LF);
          uart_send_string("Stopping timer", UART_LF);
          timer_stop();
        }
        else
          uart_send_string("Timer is not running", UART_LF);

        uart_rx_cmd  = UART_RX_CMD_NONE;
        program_mode = MSP_DEMO_MODE_STEP;
        uart_send_string(">", UART_NO_LF);
      }
      else
      if (uart_rx_cmd == UART_RX_CMD_SLEEP)
      {
        uart_send_string("", UART_LF);
        uart_send_string("Power down the IWR1443", UART_LF);
        gpio_enter_ir1443_reset();
        uart_rx_cmd  = UART_RX_CMD_NONE;
        program_mode = MSP_DEMO_MODE_STEP;
        uart_send_string(">", UART_NO_LF);
      }
      else
      if (uart_rx_cmd == UART_RX_CMD_WAKE)
      {
        uart_send_string("", UART_LF);
        uart_send_string("Power up the IWR1443", UART_LF);
        gpio_release_ir1443_reset();
        uart_rx_cmd  = UART_RX_CMD_NONE;
        program_mode = MSP_DEMO_MODE_STEP;
        uart_send_string(">", UART_NO_LF);
      }
      else
      if (uart_rx_cmd == UART_RX_CMD_CAL)
      {
          spi_init_cal();
                 usleep(10000);
        uart_send_string("", UART_LF);
        uart_send_string("Starting Calibration", UART_LF);
        uart_rx_cmd  = UART_RX_CMD_NONE;
        usleep(10000);
        program_mode = MSP_DEMO_MODE_CAL;
        usleep(10000);
        perform_calibration();
        uart_send_string(">", UART_NO_LF);
            spi_init_again();
      }
      else
      if (uart_rx_cmd == UART_RX_CMD_HELP)
      {
        uart_send_usage_info();
        uart_rx_cmd  = UART_RX_CMD_NONE;
        program_mode = MSP_DEMO_MODE_STEP;
      }
      else
        PCM_gotoLPM0();
    }
  }
}
