/*
* Copyright (c) 2017 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free,
* non-exclusive license under copyrights and patents it now or hereafter
* owns or controls to make, have made, use, import, offer to sell and sell ("Utilize")
* this software subject to the terms herein.  With respect to the foregoing patent
* license, such license is granted  solely to the extent that any such patent is necessary
* to Utilize the software alone.  The patent license shall not apply to any combinations which
* include this software, other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license (including the
* above copyright notice and the disclaimer and (if applicable) source code license limitations below)
* in the documentation and/or other materials provided with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided that the following
* conditions are met:
*
*             * No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any
*               software provided in binary form.
*             * any redistribution and use are licensed by TI for use only with TI Devices.
*             * Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the source code are permitted
* provided that the following conditions are met:
*
*   * any redistribution and use of the source code, including any resulting derivative works, are licensed by
*     TI for use only with TI Devices.
*   * any redistribution and use of any object code compiled from the source code and any resulting derivative
*     works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or
* promote products derived from this software without specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/
/*******************************************************************************
 * Add description here.
 *
 *
 * This file contains UART functions for the MSP432 side demo executable.
 *
 *******************************************************************************/
/* Standard Includes */
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#include <ti/drivers/UART.h>
#include <ti/drivers/Power.h>
#include <ti/drivers/dpl/SemaphoreP.h>

/* Demo Includes */
#include "Board.h"
#include "demo_types.h"
#include "msp_uart.h"


UART_Handle uart;
SemaphoreP_Params       uart_semParams;
SemaphoreP_Handle       uart_semHandle;

/* Globals */
char     uart_rx_buf[UART_RX_BUF_SIZE];
volatile uint16_t uart_rx_cmd = UART_RX_CMD_NONE;
uint16_t uart_rx_idx = 0;


void *uart_thread(void *arg0)
{
  char rx;

  while (1)
  {
    //UART_read will block the thread until a key is pressed.
    UART_read(uart, &rx, 1);

    //Convert uppercase chars to lowercase.
    if ((rx >= 'A') && (rx <= 'Z')) rx += 32;

    uart_rx_buf[uart_rx_idx++] = rx;
    if (uart_rx_idx == UART_RX_BUF_SIZE)
      uart_rx_idx = 0;

    //If the last command was RUN, allow any key to stop it.
    if (uart_rx_cmd == UART_RX_CMD_RUN)
    {
      uart_rx_cmd = UART_RX_CMD_STOP;
      uart_rx_idx = 0;
      return(NULL);
    }

    switch (rx)
    {
      case 8:    //backspace key
        //Remove the backspace character and the previous character
        if (uart_rx_idx > 1)
        {
          uart_rx_idx -= 2;
          rx = 32; //send a space character to erase the previous character
          UART_write(uart, &rx, 1);
          rx = 8; //send a back space character
          UART_write(uart, &rx, 1);
        }
        break;

      case 10:   //return key
        break;

      case 32:   //space bar - run or stop
        if (uart_rx_cmd == UART_RX_CMD_RUN)
          uart_rx_cmd = UART_RX_CMD_STOP;
        else
          uart_rx_cmd = UART_RX_CMD_RUN;
        uart_rx_idx = 0;
        break;

      case 'c':  //Calibrate command
        uart_rx_cmd   = UART_RX_CMD_CAL;
        uart_rx_idx = 0;
        break;

      case 'h':  //Help command
        uart_rx_cmd   = UART_RX_CMD_HELP;
        uart_rx_idx = 0;
        break;

      case 'p':  //Sleep command
        uart_rx_cmd   = UART_RX_CMD_SLEEP;
        uart_rx_idx = 0;
        break;

      case 's':  //Step command
        uart_rx_cmd   = UART_RX_CMD_STEP;
        uart_rx_idx = 0;
        break;

      case 'w':  //Wake command
        uart_rx_cmd   = UART_RX_CMD_WAKE;
        uart_rx_idx = 0;
        break;
    } //end switch

    SemaphoreP_post(uart_semHandle);
  }
}


//This function sends a string to the UART port to the PC. The input string
//must be null terminated.
void uart_send_string(char *msg, bool lf)
{
  uint32_t len;
  char     lnchar  = 10;
  char     retchar = 13;

  len = strlen(msg);

  if (len > 0)
    UART_write(uart, msg, len);

  //send "linefeed" and "return" characters.
  if (lf)
  {
    UART_write(uart, &lnchar, 1);
    UART_write(uart, &retchar, 1);
  }
}


//This function sends info to the UART port to the PC that tells the user
//the available commands that may be used.
void uart_send_usage_info(void)
{
  uart_send_string("", UART_LF);
  uart_send_string("Available commands:", UART_LF);
  uart_send_string("  c (Cal)  - run IWR1443 boot calibration", UART_LF);
  uart_send_string("  h (Help) - get this info display", UART_LF);
  uart_send_string("  p (sleeP)- put the IWR1443 into reset", UART_LF);
  uart_send_string("  s (Step) - take a single measurement", UART_LF);
  uart_send_string("  w (Wake) - take the IWR1443 out of reset", UART_LF);
  uart_send_string("  <space>  - start periodic measurements", UART_LF);
  uart_send_string("  <space>  - stop periodic measurements", UART_LF);
  uart_send_string(">", UART_NO_LF);
}


void uart_init(void)
{
  UART_Params uartParams;

  UART_init();

  /* Create a UART with data processing off. */
  UART_Params_init(&uartParams);

//  uartParams.writeDataMode = UART_DATA_TEXT;
//  uartParams.readDataMode = UART_DATA_BINARY;
//  uartParams.readMode = UART_MODE_CALLBACK;
//  uartParams.readDataMode = UART_DATA_TEXT;
//  uartParams.readReturnMode = UART_RETURN_FULL;
//  uartParams.readReturnMode = UART_RETURN_NEWLINE;
//  uartParams.readEcho = UART_ECHO_ON;
//  uartParams.baudRate = 115200;
//uartParams.readEcho = UART_ECHO_OFF;

  uart = UART_open(Board_UART0, &uartParams);

  if (uart == NULL) /* UART_open() failed */
  {
    demo_printf("ERROR: UART failed to open\n");
    while (1);
  }

  SemaphoreP_Params_init(&(uart_semParams));
  uart_semParams.mode = SemaphoreP_Mode_BINARY;

  uart_semHandle = SemaphoreP_create(0, &(uart_semParams));
  if (uart_semHandle == NULL)
  {
    demo_printf("UART: SemaphoreP_create() failed.");
  }

  uart_rx_idx = 0;
  uart_rx_cmd = UART_RX_CMD_NONE;
}
